@@ -1,14 +1,17 @@
 import logging
+import os
 import sys
 import traceback
-from argparse import ArgumentParser, ArgumentTypeError, Namespace
+from argparse import Action, ArgumentParser, ArgumentTypeError, Namespace
 from itertools import chain
 from typing import Iterator, List, Optional, Tuple, Union
 
+import shtab
+
 from dql import __version__, utils
 from dql.catalog import Catalog
 from dql.client import Client
-from dql.data_storage import DefaultDataStorage
+from dql.data_storage import SQLiteDataStorage
 from dql.node import long_line_str
 
 logger = logging.getLogger("dql")
@@ -35,8 +38,8 @@ def human_time_type(
 
 def add_sources_arg(
     parser: ArgumentParser, nargs: Union[str, int] = "+"
-) -> None:
-    parser.add_argument(
+) -> Action:
+    return parser.add_argument(
         "sources",
         type=str,
         nargs=nargs,
@@ -85,7 +88,9 @@ def get_parser() -> ArgumentParser:
     parse_get.add_argument(
         "source", type=str, help="Data source - a path to a cloud storage dir"
     )
-    parse_get.add_argument("-o", "--output", type=str, help="Output")
+    parse_get.add_argument(  # type: ignore[attr-defined]
+        "-o", "--output", type=str, help="Output"
+    ).complete = shtab.DIR
     parse_get.add_argument(
         "-f",
         "--force",
@@ -98,7 +103,7 @@ def get_parser() -> ArgumentParser:
     parse_cp = subp.add_parser(
         "cp", parents=[parent_parser], help="Copy data files from the cloud"
     )
-    add_sources_arg(parse_cp)
+    add_sources_arg(parse_cp).complete = shtab.DIR  # type: ignore[attr-defined] # noqa: E501
     parse_cp.add_argument("output", type=str, help="Output")
     parse_cp.add_argument(
         "-f",
@@ -157,6 +162,31 @@ def get_parser() -> ArgumentParser:
         "du", parents=[parent_parser], help="Display space usage"
     )
     add_sources_arg(parse_du)
+    parse_du.add_argument(
+        "-b",
+        "--bytes",
+        default=False,
+        action="store_true",
+        help="Display sizes in bytes instead of human-readable sizes",
+    )
+    parse_du.add_argument(
+        "-d",
+        "--depth",
+        "--max-depth",
+        default=0,
+        type=int,
+        metavar="N",
+        help=(
+            "Display sizes for N directory depths below the given directory, "
+            "the default is 0 (summarize provided directory only)."
+        ),
+    )
+    parse_du.add_argument(
+        "--si",
+        default=False,
+        action="store_true",
+        help="Display sizes using powers of 1000 not 1024",
+    )
 
     parse_find = subp.add_parser(
         "find", parents=[parent_parser], help="Walk a file hierarchy"
@@ -187,16 +217,32 @@ def get_parser() -> ArgumentParser:
         "--jmespath",
         type=str,
         action="append",
-        help="JMESPath query to annotation",
+        help="JMESPath query to annotation (not supported currently)",
     )
 
     parse_index = subp.add_parser(
         "index", parents=[parent_parser], help="Index storage location"
     )
     add_sources_arg(parse_index)
+    add_completion_parser(subp, [parent_parser])
     return parser
 
 
+def add_completion_parser(subparsers, parents):
+    parser = subparsers.add_parser(
+        "completion",
+        parents=parents,
+        help="Output shell completion script",
+    )
+    parser.add_argument(
+        "-s",
+        "--shell",
+        help="Shell syntax for completions.",
+        default="bash",
+        choices=shtab.SUPPORTED_SHELLS,
+    )
+
+
 def get_logging_level(args: Namespace) -> int:
     if args.quiet:
         return logging.CRITICAL
@@ -206,13 +252,13 @@ def get_logging_level(args: Namespace) -> int:
 
 
 def get(source, output, **kwargs):
-    data_storage = DefaultDataStorage()
+    data_storage = SQLiteDataStorage()
     catalog = Catalog(data_storage)
     catalog.get(source, output, **kwargs)
 
 
 def cp(sources, output, **kwargs):
-    data_storage = DefaultDataStorage()
+    data_storage = SQLiteDataStorage()
     catalog = Catalog(data_storage)
     catalog.cp(sources, output, **kwargs)
 
@@ -228,7 +274,7 @@ def ls_urls(
     if client_config is None:
         client_config = {}
     if catalog is None:
-        catalog = Catalog(DefaultDataStorage())
+        catalog = Catalog(SQLiteDataStorage())
 
     curr_dir = None
     value_iterables = []
@@ -278,7 +324,7 @@ def ls_indexed_storages(
     long: bool = False, catalog: Optional[Catalog] = None
 ) -> Iterator[str]:
     if catalog is None:
-        catalog = Catalog(DefaultDataStorage())
+        catalog = Catalog(SQLiteDataStorage())
     storages = catalog.ls_storages()
     if long:
         for s in storages:
@@ -293,7 +339,7 @@ def ls(
     sources, long: bool = False, catalog: Optional[Catalog] = None, **kwargs
 ):
     if catalog is None:
-        catalog = Catalog(DefaultDataStorage())
+        catalog = Catalog(SQLiteDataStorage())
     if sources:
         if len(sources) == 1:
             for _, entries in ls_urls(
@@ -320,25 +366,35 @@ def ls(
             print(entry)
 
 
-def du(sources, **kwargs):
-    data_storage = DefaultDataStorage()
+def du(sources, show_bytes=False, si=False, **kwargs):
+    data_storage = SQLiteDataStorage()
     catalog = Catalog(data_storage)
     for path, size in catalog.du(sources, **kwargs):
-        print(f"{utils.sizeof_fmt(size): >7} {path}")
+        if show_bytes:
+            print(f"{size} {path}")
+        else:
+            print(f"{utils.sizeof_fmt(size, si=si): >7} {path}")
 
 
 def find(sources, **kwargs):
-    data_storage = DefaultDataStorage()
+    data_storage = SQLiteDataStorage()
     catalog = Catalog(data_storage)
     yield from catalog.find(sources, **kwargs)
 
 
 def index(sources, catalog: Optional[Catalog] = None, **kwargs):
     if catalog is None:
-        catalog = Catalog(DefaultDataStorage())
+        catalog = Catalog(SQLiteDataStorage())
     catalog.index(sources, **kwargs)
 
 
+def completion(shell: str) -> str:
+    return shtab.complete(
+        get_parser(),
+        shell=shell,  # nosec B604
+    )
+
+
 def main(argv: Optional[List[str]] = None) -> int:  # noqa: C901
     parser = get_parser()
     args = parser.parse_args(argv)
@@ -391,11 +447,15 @@ def main(argv: Optional[List[str]] = None) -> int:  # noqa: C901
         elif args.command == "du":
             du(
                 args.sources,
+                show_bytes=args.bytes,
+                depth=args.depth,
+                si=args.si,
                 ttl=args.ttl,
                 update=bool(args.update),
                 client_config=client_config,
             )
         elif args.command == "find":
+            results_found = False
             for result in find(
                 args.sources,
                 ttl=args.ttl,
@@ -407,6 +467,9 @@ def main(argv: Optional[List[str]] = None) -> int:  # noqa: C901
                 client_config=client_config,
             ):
                 print(result)
+                results_found = True
+            if not results_found:
+                print("No results")
         elif args.command == "index":
             index(
                 args.sources,
@@ -414,7 +477,19 @@ def main(argv: Optional[List[str]] = None) -> int:  # noqa: C901
                 update=bool(args.update),
                 client_config=client_config,
             )
+        elif args.command == "completion":
+            print(completion(args.shell))
+        else:
+            print(f"invalid command: {args.command}", file=sys.stderr)
+            return 1
         return 0
+    except BrokenPipeError:
+        # Python flushes standard streams on exit; redirect remaining output
+        # to devnull to avoid another BrokenPipeError at shutdown
+        # See: https://docs.python.org/3/library/signal.html#note-on-sigpipe
+        devnull = os.open(os.devnull, os.O_WRONLY)
+        os.dup2(devnull, sys.stdout.fileno())
+        return 141  # 128 + 13 (SIGPIPE)
     except Exception as exc:  # pylint: disable=broad-except
         print("Error:", exc, file=sys.stderr)
         if logging_level <= logging.DEBUG:
