@@ -18,10 +18,10 @@ class GCSClient(FSSpecClient):
         super().__init__(name, self.create_fs(**kwargs))
 
     @classmethod
-    def _df_from_info(cls, v, dir_id, delimiter, path):
+    def _dict_from_info(cls, v, parent_id, delimiter, path):
         return {
-            "dir_id": None,
-            "parent_id": dir_id,
+            "is_dir": False,
+            "parent_id": parent_id,
             "path": path,
             "name": v.get("name", "").split(delimiter)[-1],
             # 'expires': expires,
