@@ -4,7 +4,7 @@ from datetime import datetime
 from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple
 
 from dql.node import Node, NodeWithPath
-from dql.storage import Storage
+from dql.storage import AbstractStorage, Storage
 
 DB_FILE_NAME = "db"
 
@@ -14,7 +14,7 @@ logger = logging.getLogger("dql")
 
 class AbstractDataStorage(ABC):
     @abstractmethod
-    def clone(self, storage: Storage) -> "AbstractDataStorage":
+    def clone(self, uri: Optional[str]) -> "AbstractDataStorage":
         """Clones DataStorage implementation for some Storage input"""
 
     @abstractmethod
@@ -22,31 +22,76 @@ class AbstractDataStorage(ABC):
         """Initializes database tables for data storage"""
 
     @abstractmethod
-    async def insert(self, values: List[Dict[str, Any]], **kwargs) -> None:
-        """Inserts file or directory node into the database"""
+    async def insert_entry(self, entry: Dict[str, Any]) -> int:
+        """
+        Inserts file or directory node into the database
+        and returns the id of the newly added node
+        """
 
     @abstractmethod
-    async def get_dir(self, parent_id: str, name: str) -> Optional[str]:
-        """Gets directory id from database by name and parent_id"""
+    async def insert_entries(self, entries: List[Dict[str, Any]]) -> None:
+        """Inserts file or directory nodes into the database"""
 
     @abstractmethod
-    def upsert_storage(self, storage: Storage) -> None:
+    async def insert_root(self) -> int:
+        """
+        Inserts root directory and returns the id of the newly added root
+        """
+
+    @abstractmethod
+    def get_node_by_id(self, node_id: int) -> Optional[NodeWithPath]:
+        """Gets node from database by id"""
+
+    @abstractmethod
+    def get_nodes_by_id(
+        self, node_ids: Iterable[int]
+    ) -> Iterable[NodeWithPath]:
+        """Gets nodes from database by id"""
+
+    @abstractmethod
+    def get_node_by_name_and_parent(
+        self, name: str, parent_id: int
+    ) -> Optional[NodeWithPath]:
+        """Gets node from database by name and parent_id"""
+
+    @abstractmethod
+    def get_nodes_by_parent_id(
+        self,
+        parent_id: int,
+        type_filter: Optional[str] = None,
+    ) -> Iterable[NodeWithPath]:
+        """Gets nodes from database by parent_id, with optional filtering"""
+
+    @abstractmethod
+    def upsert_storage(self, uri: str) -> AbstractStorage:
         """
         Saves new storage if it doesn't exist in database, otherwise it updates
         basic fields
         """
 
     @abstractmethod
-    def register_storage_for_indexing(self, storage: Storage) -> None:
+    def mark_storage_pending(self, storage: Storage) -> Storage:
+        """
+        Sets storage in pending state, which means it's ready for indexing
+        """
+
+    @abstractmethod
+    def register_storage_for_indexing(
+        self, uri: str, force_update: bool
+    ) -> Tuple[Storage, bool, bool]:
         """
         Prepares storage for indexing operation.
         This method should be called before index operation is started
+        It returns:
+            - storage, prepared for indexing
+            - boolean saying if indexing is needed
+            - boolean saying if indexing is currently pending (running)
         """
 
     @abstractmethod
     def mark_storage_indexed(
         self,
-        storage_id: str,
+        uri: str,
         status: int,
         ttl: int,
         end_time: Optional[datetime] = None,
@@ -56,32 +101,22 @@ class AbstractDataStorage(ABC):
         This method should be called when index operation is finished
         """
 
-    @abstractmethod
-    async def insert_root(self, storage: Storage) -> int:
-        """
-        Inserts root directory for some storage.
-        It returns directory id of a newly created root
-        """
-
     @abstractmethod
     def get_storage_all(self) -> Iterator[Storage]:
         pass
 
     @abstractmethod
-    def get_storage(
-        self, storage_id: str, expires_at: str
-    ) -> Optional[Storage]:
+    def get_storage(self, uri: str) -> Optional[Storage]:
         """
         Gets storage representation from database.
         E.g if s3 is used as storage this would be s3 bucket data
         """
 
     @abstractmethod
-    def size(self, node: NodeWithPath, inames=None) -> Tuple[float, int]:
+    def size(self, node: NodeWithPath) -> Tuple[int, int]:
         """
         Calculates size of some node (and subtree below node).
-        It can be filtered out by inames.
-        Returns size in bytes as float and total files as int
+        Returns size in bytes as int and total files as int
         """
 
     @abstractmethod
@@ -96,42 +131,33 @@ class AbstractDataStorage(ABC):
         path is strict (without GLOB patterns) only one node will be returned.
         """
 
+    @abstractmethod
+    def walk_subtree(
+        self,
+        node: NodeWithPath,
+        sort_by_parent_id: bool = False,
+        sort_by_size: bool = False,
+        sort_by_path: bool = False,
+        type_filter: Optional[str] = None,
+        custom_filter: Optional[str] = None,
+    ) -> Iterable[NodeWithPath]:
+        """
+        Returns all directory and file nodes that are "below" some node.
+        Nodes can be sorted or filtered as well.
+        """
+
     @abstractmethod
     def get_latest_files_by_parent_node(
         self, parent_node: Node
     ) -> Iterable[Node]:
         """
-        Gets file nodes that are latest version and have some parent node
-        """
-
-    @abstractmethod
-    def walk_subtree(
-        self,
-        node: Node,
-        filter="",  # pylint: disable=redefined-builtin
-        jmespath="",
-        sort_by_size=False,
-        sort_by_name=False,
-    ) -> Iterable[NodeWithPath]:
-        """
-        Returns all directory and file nodes that are "below" some node.
-        Filters can be used for filtering some properties in nodes and jmespath
-        as well.
-        """
-
-    @abstractmethod
-    def walk_subtree_files_sorted_by_dir(
-        self, node: NodeWithPath
-    ) -> Iterable[NodeWithPath]:
-        """
-        Returns all directory and file nodes that are "below" some node
-        sorted by parent ids
+        Gets latest-version file nodes from the provided parent node
         """
 
     @abstractmethod
     def find(
         self,
-        node: Node,
+        node: NodeWithPath,
         names=None,
         inames=None,
         type=None,  # pylint: disable=redefined-builtin
