@@ -12,7 +12,7 @@ from tqdm import tqdm
 
 from dql.client import Client
 from dql.data_storage import AbstractDataStorage
-from dql.node import Node, NodeWithPath
+from dql.node import DirType, Node, NodeWithPath
 from dql.nodes_fetcher import JsonNodesFetcher
 from dql.storage import Storage
 
@@ -32,7 +32,6 @@ class Listing:
         data_storage: AbstractDataStorage,
         client: Client,
     ):
-        self.db = None
         self.storage = storage
         self.data_storage = data_storage
         self.client = client
@@ -46,45 +45,50 @@ class Listing:
 
     @staticmethod
     async def _insert_dir(data_storage, parent_id, name, time, path):
-        await data_storage.insert(
-            [
-                {
-                    "parent_id": parent_id,
-                    "path": path,
-                    "name": name,
-                    "last_modified": time,
-                    "checksum": "",
-                    "etag": "",
-                    "version": "",
-                    "is_latest": True,
-                    "size": 0,
-                    "owner_name": "",
-                    "owner_id": "",
-                },
-            ],
-            is_dir=True,
+        return await data_storage.insert_entry(
+            {
+                "is_dir": True,
+                "parent_id": parent_id,
+                "path": path,
+                "name": name,
+                "last_modified": time,
+                "checksum": "",
+                "etag": "",
+                "version": "",
+                "is_latest": True,
+                "size": 0,
+                "owner_name": "",
+                "owner_id": "",
+            },
         )
-        new_dir_id = await data_storage.get_dir(parent_id, name)
-        return new_dir_id
 
-    async def insert_dir(self, parent_id, name, time, path):
+    async def insert_dir(self, parent_id, name, time, path, data_storage=None):
         return await Listing._insert_dir(
-            self.data_storage,
+            data_storage or self.data_storage,
             parent_id,
             name,
             time,
             path,
         )
 
-    async def insert_file(self, parent_id, name, time):
-        node = Node(None, parent_id, name, time)
+    async def insert_file(self, parent_id, name, time, path_str):
+        node = Node(
+            0,
+            DirType.FILE,
+            parent_id,
+            name,
+            last_modified=time,
+            path_str=path_str,
+        )
         await self.insert_file_from_node(node)
 
     async def insert_file_from_node(self, node):
-        await self.data_storage.insert([node._asdict()], is_dir=node.is_dir)
+        await self.data_storage.insert_entry(
+            {**node._asdict(), "is_dir": False}
+        )
 
-    async def insert_root(self) -> int:
-        return await self.data_storage.insert_root(self.storage)
+    async def insert_root(self, data_storage=None) -> int:
+        return await (data_storage or self.data_storage).insert_root()
 
     def expand_path(self, path) -> Iterable[NodeWithPath]:
         return self.data_storage.get_nodes_by_path(path)
@@ -144,8 +148,11 @@ class Listing:
                 dir_path = node_path[:-1]
                 if not copy_dir_contents:
                     dir_path.append(node.name)
-                subtree_nodes = (
-                    self.data_storage.walk_subtree_files_sorted_by_dir(node)
+                subtree_nodes = self.data_storage.walk_subtree(
+                    node,
+                    sort_by_parent_id=True,
+                    sort_by_path=True,
+                    type_filter="files",
                 )
                 all_nodes.extend(
                     n._replace(path=dir_path + n.path) for n in subtree_nodes
@@ -266,8 +273,7 @@ class Listing:
             else:
                 group_path_elements = []
             for node in updated_nodes:
-                combined_path = "/".join(group_path_elements + [node.name])
-                updated_node_lookup[combined_path] = node._replace(
+                updated_node_lookup[node.id] = node._replace(
                     path=group_path_elements
                 )
 
@@ -275,10 +281,9 @@ class Listing:
         # after they are downloaded
         updated_nodes = []
         for node in nodes:
-            combined_path = "/".join(node.path + [node.name])
-            if combined_path in updated_node_lookup:
+            if node.id in updated_node_lookup:
                 # Node has updated information
-                updated_nodes.append(updated_node_lookup[combined_path])
+                updated_nodes.append(updated_node_lookup[node.id])
             else:
                 # No updates
                 updated_nodes.append(node)
@@ -305,5 +310,5 @@ class Listing:
             node, names=names, inames=inames, type=type, jmespath=jmespath
         )
 
-    def du(self, node, inames=None):
-        return self.data_storage.size(node, inames=inames)
+    def du(self, node):
+        return self.data_storage.size(node)
