@@ -6,9 +6,16 @@ from typing import List, NamedTuple, Optional
 from dql.utils import time_to_str
 
 
+class DirType:
+    FILE = 0
+    DIR = 1
+    ROOT = 2
+
+
 class NodeRecord(NamedTuple):
-    dir_id: Optional[str] = None
-    parent_id: Optional[str] = None
+    id: int = 0
+    dir_type: Optional[int] = None
+    parent_id: Optional[int] = None
     name: Optional[str] = None
     checksum: str = ""
     etag: str = ""
@@ -18,6 +25,7 @@ class NodeRecord(NamedTuple):
     size: int = 0
     owner_name: str = ""
     owner_id: str = ""
+    path_str: str = ""
     anno: Optional[str] = None
 
 
@@ -33,36 +41,38 @@ class AbstractNode:
     TIME_FMT = "%Y-%m-%d %H:%M"
 
     @property
-    def is_dir(self):
-        return self.dir_id is not None
+    def id(self) -> int:
+        return self.id
 
     @property
-    def dir_id(self):
-        return self.dir_id
+    def is_dir(self) -> bool:
+        return bool(self.dir_type)
 
     @property
-    def name(self):
+    def dir_type(self) -> Optional[int]:
+        return self.dir_type
+
+    @property
+    def path_str(self) -> str:
+        return self.path_str
+
+    @property
+    def name(self) -> Optional[str]:
         return self.name
 
     @property
-    def name_no_ext(self):
+    def name_no_ext(self) -> str:
+        if not self.name:
+            return ""
         name, _ = os.path.splitext(self.name)
         return name
 
     @property
-    def dir_id_sql(self):
-        return self.dir_id if (self.dir_id is not None) else "NULL"
+    def is_downloadable(self) -> bool:
+        return bool(not self.is_dir and self.name)
 
     @property
-    def equalis_dir_id_sql(self):
-        return f" = {self.dir_id}" if (self.dir_id is not None) else " IS NULL"
-
-    @property
-    def is_downloadable(self):
-        return not self.is_dir and self.name
-
-    @property
-    def size(self):
+    def size(self) -> int:
         return self.size or 0
 
     def get_metafile_data(self):
@@ -133,7 +143,7 @@ class NodeWithPath(  # pylint:disable=inherit-non-class
     pass
 
 
-def long_line_str(name: str, timestamp: Optional[datetime], owner: str):
+def long_line_str(name: str, timestamp: Optional[datetime], owner: str) -> str:
     if timestamp is None:
         time = "-"
     else:
