@@ -3,6 +3,8 @@ import os.path as osp
 import stat
 from typing import Iterable, Optional, Type, TypeVar
 
+from dateutil.parser import isoparse
+
 GLOB_CHARS = ["?", "*", "[", "]"]
 
 T = TypeVar("T", bound="DQLDir")
@@ -83,17 +85,20 @@ def human_time_to_int(time: str) -> Optional[int]:
 
 
 def time_to_str(dt):
+    if isinstance(dt, str):
+        dt = isoparse(dt)
     return dt.strftime("%Y-%m-%d %H:%M:%S")
 
 
-def sizeof_fmt(num, suffix=""):
-    for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
-        if abs(num) < 1024.0:
+def sizeof_fmt(num, suffix="", si=False):
+    power = 1000.0 if si else 1024.0
+    for unit in ["", "K", "M", "G", "T", "P", "E", "Z", "Y", "R"]:
+        if abs(num) < power:
             if not unit:
                 return f"{num:4.0f}{suffix}"
             return f"{num:3.1f}{unit}{suffix}"
-        num /= 1024.0
-    return f"{num:.1f}Yi{suffix}"
+        num /= power
+    return f"{num:.1f}Q{suffix}"
 
 
 def force_create_dir(name):
@@ -119,15 +124,15 @@ def sql_file_filter(
         [f"name {op} {sql_str(name)}" for name in names] if names else []
     )
     inames_sql = (
-        [f"lcase(name) {op} lcase({sql_str(iname)}')" for iname in inames]
+        [f"LOWER(name) {op} ({sql_str(iname.lower())})" for iname in inames]
         if inames
         else []
     )
     name_filter = " OR ".join(names_sql + inames_sql)
 
     type_filter = {
-        "f": "dir_id IS NULL",
-        "d": "dir_id IS NOT NULL",
+        "f": "dir_type = 0",
+        "d": "dir_type > 0",
         "": "",
         None: "",
     }.get(type, None)
