@@ -618,6 +618,35 @@ def test_get_subdir(cloud_test_catalog):
         )
 
 
+@pytest.mark.parametrize("cloud_test_catalog", ["s3", "gcs"], indirect=True)
+def test_du(cloud_test_catalog):
+    src = str(cloud_test_catalog.src)
+    catalog = cloud_test_catalog.catalog
+
+    expected_results = [
+        (f"{src}cats/", 8),
+        (f"{src}dogs/others/", 4),
+        (f"{src}dogs/", 15),
+        (src, 36),
+    ]
+
+    results = catalog.du([src], client_config=cloud_test_catalog.client_config)
+
+    assert list(results) == expected_results[3:]
+
+    results = catalog.du(
+        [src], client_config=cloud_test_catalog.client_config, depth=1
+    )
+
+    assert list(results) == expected_results[:1] + expected_results[2:]
+
+    results = catalog.du(
+        [src], client_config=cloud_test_catalog.client_config, depth=5
+    )
+
+    assert list(results) == expected_results
+
+
 @pytest.mark.parametrize("cloud_test_catalog", ["s3", "gcs"], indirect=True)
 def test_ls_glob(cloud_test_catalog):
     src = cloud_test_catalog.src
